package tech.mercantec.easyeat.ui.dishes

import android.app.Activity
import android.app.ProgressDialog
import kotlinx.serialization.json.Json
import android.os.Bundle
import android.view.LayoutInflater
import android.widget.ArrayAdapter
import android.widget.Button
import android.widget.EditText
import android.widget.ImageButton
import android.widget.LinearLayout
import android.widget.Spinner
import android.widget.Toast
import androidx.appcompat.app.AppCompatActivity
import tech.mercantec.easyeat.R
import tech.mercantec.easyeat.helpers.ApiRequestException
import tech.mercantec.easyeat.helpers.createRecipe
import tech.mercantec.easyeat.helpers.updateRecipe
import tech.mercantec.easyeat.models.Ingredient
import tech.mercantec.easyeat.models.Recipe
import kotlin.concurrent.thread

class EditDishActivity : AppCompatActivity() {

    private lateinit var ingredientContainer: LinearLayout

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_dish_form)

        val recipe: Recipe? = intent.getStringExtra("recipe_json")?.let {
            try {
                Json.decodeFromString<Recipe>(it)
            } catch (e: Exception) {
                null
            }
        }

        val dishName = findViewById<EditText>(R.id.dishName)
        val dishDescription = findViewById<EditText>(R.id.dishDescription)
        val instructions = findViewById<EditText>(R.id.instructions)
        val saveButton = findViewById<Button>(R.id.saveDishButton)
        ingredientContainer = findViewById(R.id.ingredientContainer)

        // Populate UI
        dishName.setText(recipe?.name)
        dishDescription.setText(recipe?.description)
        instructions.setText(recipe?.directions?.joinToString("\n"))

        for (ingredient in recipe?.ingredients!!) {
            addIngredientRow(ingredient)
        }

        findViewById<Button>(R.id.addIngredientButton).setOnClickListener {
            addIngredientRow()
        }

            saveButton.setOnClickListener {
                val ingredientList = collectIngredients()
                val instructions = findViewById<EditText>(R.id.instructions).text.toString().trim()
                val directions: List<String> = instructions
                    .split("\n")
                    .map { line -> line.trim() }
                    .filter { it.isNotEmpty() }

                val updatedRecipe = Recipe(
                    name = findViewById<EditText>(R.id.dishName).text.toString().trim(),
                    description = findViewById<EditText>(R.id.dishDescription).text.toString().trim(),
                    directions = directions,
                    ingredients = ingredientList
                )

            val progressDialog = ProgressDialog(this)
            progressDialog.setMessage("Loading...")
            progressDialog.show()
            thread {
                try {

                    updateRecipe(this, recipe.id!!, updatedRecipe)
                } catch (e: ApiRequestException) {
                    runOnUiThread {
                        Toast.makeText(this, e.message, Toast.LENGTH_LONG).show()
                    }

                    return@thread
                } finally {
                    runOnUiThread {
                        progressDialog.hide()
                    }
                }

                runOnUiThread {
                    Toast.makeText(this, "Password changed successfully", Toast.LENGTH_LONG).show()
                }

                setResult(Activity.RESULT_OK)
                finish()
            }
        }
    }

    private fun addIngredientRow(ingredient: Ingredient? = null) {
        val inflater = LayoutInflater.from(this)
        val ingredientRow = inflater.inflate(R.layout.activity_dish_ingredient_row, null)
        val nameField = ingredientRow.findViewById<EditText>(R.id.ingredientNameEditText)
        val amountField = ingredientRow.findViewById<EditText>(R.id.ingredientAmountEditText)
        val unitSpinner = ingredientRow.findViewById<Spinner>(R.id.measurementsDropDown)

        unitSpinner.adapter = ArrayAdapter(
            this,
            android.R.layout.simple_spinner_item,
            listOf("g", "kg", "ml", "l", "tsp", "tbsp", "cup", "pcs", "pinch", "dash")
        )

        if (ingredient != null) {
            nameField.setText(ingredient.name)
            amountField.setText(ingredient.amount.toString())
            unitSpinner.setSelection((unitSpinner.adapter as ArrayAdapter<String>).getPosition(ingredient.unit))
        }

        ingredientRow.findViewById<ImageButton>(R.id.removeButton).setOnClickListener {
            ingredientContainer.removeView(ingredientRow)
        }

        ingredientContainer.addView(ingredientRow)
    }
    private fun collectIngredients(): List<Ingredient> {
        val ingredients = mutableListOf<Ingredient>()

        for (i in 0 until ingredientContainer.childCount) {
            val ingredientView = ingredientContainer.getChildAt(i)

            // Find views inside this ingredient row

            val ingredientEditText = ingredientView.findViewById<EditText>(R.id.ingredientNameEditText)
            val amountEditText = ingredientView.findViewById<EditText>(R.id.ingredientAmountEditText)
            val spinner = ingredientView.findViewById<Spinner>(R.id.measurementsDropDown)

            val element = ingredientEditText.text.toString().trim()
            val amount = amountEditText.text.toString().trim()
            val unit = spinner.selectedItem.toString()

            // Optional: Only add non-empty rows
            if (element.isNotEmpty() && amount.isNotEmpty()) {
                ingredients.add(Ingredient(name = element, amount = amount.toDouble(), unit = unit))
            }
        }

        return ingredients
    }
}
